/**
 * Abstract class that implements the basic functionality of a frog,
 * and represents it using simple graphical elements.
 * 
 * @author CS1711 Instructors
 * @version 2005-11-05
 */
public abstract class Frog
{
    // Frog size constants
    public static final int HEADHEIGHT = 15;
    public static final int HEADWIDTH = 41;
    public static final int EYESIZE = 11;
    public static final String EYECOLOR = "black";

    // Constants for edge constraints
    public static final int CANVASMAXRIGHT = 499;
    public static final int CANVASMAXDOWN = 499;    
    public static final int LEFTWALL = HEADWIDTH/2;
    public static final int RIGHTWALL = CANVASMAXRIGHT - HEADWIDTH/2;
    public static final int TOPWALL = 0;
    public static final int BOTTOMWALL = CANVASMAXDOWN - HEADHEIGHT;  

    // Constants for initial Triangle, Circle, and Frog positions
    public static final int INIT_TRIANGLE_X = 50;
    public static final int INIT_TRIANGLE_Y = 15;
    public static final int INIT_CIRCLE_X = 20;
    public static final int INIT_CIRCLE_Y = 60;
    public static final int INIT_FROG_X = 150;
    public static final int INIT_FROG_Y = 100;
    public static final int HORIZ_EYE_OFFSET = 10;

    // Non-constant fields
    protected Triangle head;
    protected Circle leftEye, rightEye;
    private int vPos, hPos;

    //---------------------------------------------------------------------------

    /**
     * Constructor for default objects of class Frog.
     */
    public Frog()
    {
        head = new Triangle();
        head.changeSize(HEADHEIGHT, HEADWIDTH);

        // Move head from initial Triangle position to default position
        head.moveHorizontal(INIT_FROG_X - INIT_TRIANGLE_X);
        head.moveVertical(INIT_FROG_Y - INIT_TRIANGLE_Y);
        hPos = INIT_FROG_X;
        vPos = INIT_FROG_Y;

        leftEye = new Circle();
        leftEye.changeSize(EYESIZE);
        leftEye.changeColor(EYECOLOR);

        // Move left eye from initial Circle position to default position
        leftEye.moveHorizontal(INIT_FROG_X - INIT_CIRCLE_X - HORIZ_EYE_OFFSET - EYESIZE/2);
        leftEye.moveVertical(INIT_FROG_Y - INIT_CIRCLE_Y);

        rightEye = new Circle();
        rightEye.changeSize(EYESIZE);
        rightEye.changeColor(EYECOLOR);

        // Move right eye from initial Circle position to default position
        rightEye.moveHorizontal(INIT_FROG_X - INIT_CIRCLE_X + HORIZ_EYE_OFFSET - EYESIZE/2);
        rightEye.moveVertical(INIT_FROG_Y - INIT_CIRCLE_Y);
        makeFrogVisible();
    } // constructor Frog()

    //---------------------------------------------------------------------------

    /**
     * Constructor with parameters for objects of class Frog.
     * 
     * @param inHoriz  initial horizontal position
     * @param inVert  initial vertical position
     */
    public Frog(int inHoriz, int inVert)
    {
        // Set the position if inputs are valid, otherwise use default values
        if ((inHoriz >= LEFTWALL) && (inHoriz <= RIGHTWALL) && (inVert >= TOPWALL) && (inVert <= BOTTOMWALL))
        {
            hPos = inHoriz;
            vPos = inVert;
        } // if
        else
        {
            System.out.println("Frog partly off page -- using default position");
            hPos = INIT_FROG_X;
            vPos = INIT_FROG_Y;
        } // else

        head = new Triangle();
        head.changeSize(HEADHEIGHT, HEADWIDTH);

        // Move head from initial Triangle position to specified position
        head.moveHorizontal(hPos - INIT_TRIANGLE_X);
        head.moveVertical(vPos - INIT_TRIANGLE_Y);

        leftEye = new Circle();
        leftEye.changeSize(EYESIZE);
        leftEye.changeColor("black");

        // Move left eye from initial Circle position to specified position
        leftEye.moveHorizontal(hPos - INIT_CIRCLE_X - HORIZ_EYE_OFFSET - EYESIZE/2);
        leftEye.moveVertical(vPos - INIT_CIRCLE_Y);

        rightEye = new Circle();
        rightEye.changeSize(EYESIZE);
        rightEye.changeColor("black");

        // Move right eye from initial Circle position to specified position
        rightEye.moveHorizontal(hPos - INIT_CIRCLE_X + HORIZ_EYE_OFFSET - EYESIZE/2);
        rightEye.moveVertical(vPos - INIT_CIRCLE_Y);
        makeFrogVisible();
    } // constructor Frog(String,int,int)

    //---------------------------------------------------------------------------

    /**
     * Makes the frog visible.
     */
    public void makeFrogVisible()
    {
        head.makeVisible();
        leftEye.makeVisible();
        rightEye.makeVisible();
    } // makeFrogVisible()

    //---------------------------------------------------------------------------

    /**
     * Makes the frog invisible.
     */
    public void makeFrogInvisible()
    {
        head.makeInvisible();
        leftEye.makeInvisible();
        rightEye.makeInvisible();
    } // makeFrogInvisible()

    //---------------------------------------------------------------------------

    public abstract void gridMove(int direction);

    //---------------------------------------------------------------------------

    /**
     * Moves the frog left by 'dist' pixels, stopping at left wall if 'dist' is too large.
     * 
     * @param dist the distance to move left -- must be positive to have effect
     */
    public void frogLeft(int dist)
    {
        int actualDist;
        if (dist > 0)
        {
            actualDist = dist;
            if ((hPos - LEFTWALL) < dist)
            {
                actualDist = hPos - LEFTWALL;
            } // if
            head.moveHorizontal(-actualDist);
            leftEye.moveHorizontal(-actualDist);
            rightEye.moveHorizontal(-actualDist);
            hPos = hPos - actualDist;
        } // if
    } // frogLeft(int)

    //---------------------------------------------------------------------------

    /**
     * Moves the frog right by 'dist' pixels, stopping at right wall if 'dist' is too large.
     * 
     * @param dist the distance to move right -- must be positive to have effect
     */
    public void frogRight(int dist)
    {
        int actualDist;
        if (dist > 0)
        {
            actualDist = dist;
            if ((hPos + dist) > RIGHTWALL)
            {
                actualDist = RIGHTWALL - hPos;
            } // if
            head.moveHorizontal(actualDist);
            leftEye.moveHorizontal(actualDist);
            rightEye.moveHorizontal(actualDist);
            hPos = hPos + actualDist;
        } // if
    } // frogRight(int)

    //---------------------------------------------------------------------------

    /**
     * Moves the frog up by 'dist' pixels, stopping at top wall if 'dist' is too large.
     * 
     * @param dist the distance to move up -- must be positive to have effect
     */
    public void frogUp(int dist)
    {
        int actualDist;
        if (dist > 0)
        {
            actualDist = dist;
            if ((vPos - TOPWALL) < dist)
            {
                actualDist = vPos - TOPWALL;
            } // if
            head.moveVertical(-actualDist);
            leftEye.moveVertical(-actualDist);
            rightEye.moveVertical(-actualDist);
            vPos = vPos - actualDist;
        } // if
    } // frogUp(int)

    //---------------------------------------------------------------------------

    /**
     * Moves the frog down by 'dist' pixels, stopping at bottom wall if 'dist' is too large.
     * 
     * @param dist the distance to move down -- must be positive to have effect
     */
    public void frogDown(int dist)
    {
        int actualDist;
        if (dist > 0)
        {
            actualDist = dist;
            if ((vPos + dist) > BOTTOMWALL)
            {
                actualDist = BOTTOMWALL - vPos;
            } // if
            head.moveVertical(actualDist);
            leftEye.moveVertical(actualDist);
            rightEye.moveVertical(actualDist);
            vPos = vPos + actualDist;
        } // if
    } // frogDown(int)

    //---------------------------------------------------------------------------

}  // class Frog
